# -*- coding: utf-8 -*-
#
# SPDX-FileCopyrightText: 2021 The octopy-energy Authors
# SPDX-License-Identifier: 0BSD

# Ensure it's present.
import setuptools_scm  # noqa: F401
from setuptools import setup

setup()
